
// (c) COPYRIGHT URI/MIT 1996
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//      jhrg,jimg       James Gallagher (jgallagher@gso.uri.edu)

// Implementation for the class ClientStructure. See ClientByte.cc
//
// jhrg 1/12/95

// $Log: ClientStructure.cc,v $
// Revision 1.15.12.1  2000/06/02 23:03:32  jimg
// Fixes for AttrTables with no entries.
// Added support for name translation and a DODS_ML_Real_Name attribute.
// Changed the `size' attribtue's name to DODS_ML_Size.
//
// Revision 1.15  1999/03/24 06:23:43  brent
// convert String.h to std lib <string>, convert to packages regex -- B^2
//
// Revision 1.14  1998/11/20 08:41:22  jimg
// Fixed the fix for the extra newline separators; it was broken for arrays
//
// Revision 1.13  1998/11/19 20:53:33  jimg
// Fixed the extra newline bug.
//
// Revision 1.12  1998/11/05 21:40:58  jimg
// Fixed a bug where print_val() on structure variables was not called with
// print_decl_p passed. Instead the member function was called with the flag
// forced to false. This made it impossible for loaddods to read the the data
// stream.
//
// Revision 1.11  1998/09/16 23:02:22  jimg
// Removed write_val() and replaced it with print_val() and print_all_vals()
//
// Revision 1.10  1998/08/03 16:39:51  jimg
// Fixed write_val mfunc so that outermost is no longer used
//
// Revision 1.9  1997/10/04 00:40:08  jimg
// Added changes to support the new deserialize() mfunc and write_val().
//
// Revision 1.8  1997/04/19 00:54:59  jimg
// Changed ouput stream to simplify reading (for loaddods).
//
// Revision 1.7  1997/02/06 20:43:41  jimg
// Fixed log messages
//
// Revision 1.6  1997/01/10 06:49:38  jimg
// Changed call to name_map::lookup() so that non-alphanumerics are mapped to
// underscore.
//
// Revision 1.5  1996/11/23 05:12:14  jimg
// Added support for variable renaming via the name_map object.
//
// Revision 1.4  1996/10/25 23:13:59  jimg
// Changed/Fixed write_val() so that it uses smart_newline().
//
// Revision 1.3  1996/10/23 23:46:45  jimg
// Modified so that writeval outputs a `recursive' data stream. Thus any
// data type can be written and later read without being flattened. This
// change was made so that the NSCAT data which is represented as a Sequence
// of Structures.
//
// Revision 1.2  1996/10/07 21:00:30  jimg
// Fixed Headers.
//
// Revision 1.1  1996/09/30 23:59:21  jimg
// Added.

#ifdef _GNUG_
#pragma implementation
#endif

#include <assert.h>
#include <iostream.h>
#include <string>

#include <Pix.h>
#include <SLList.h>

#include "ClientStructure.h"
#include "ClientSequence.h"

#include "name_map.h"

extern name_map names;

extern void smart_newline(ostream &os, Type type);

Structure *
NewStructure(const string &n)
{
    return new ClientStructure(n);
}

BaseType *
ClientStructure::ptr_duplicate()
{
    return new ClientStructure(*this);
}

ClientStructure::ClientStructure(const string &n) : Structure(n)
{
}

ClientStructure::~ClientStructure()
{
}

// For this `Client' class, run the read mfunc for each of variables which
// comprise the structure. 

bool
ClientStructure::read(const string &, int &)
{
    assert(false);
    return false;
}

void 
ClientStructure::print_val(ostream &os, string space, bool print_decls)
{
    for (Pix p = first_var(); p; next_var(p)) {
	var(p)->print_val(os, "", print_decls);
    }
}

void
ClientStructure::print_all_vals(ostream &os, XDR *src, DDS *dds, string space, 
				bool print_decl_p)
{
    bool sequence_found = false;

    for (Pix p = first_var(); p; next_var(p)) {
	assert(var(p));

	switch (var(p)->type()) {
	  case dods_sequence_c:
	    (dynamic_cast<ClientSequence *>
	     (var(p)))->print_all_vals(os, src, dds, "", print_decl_p);
	    sequence_found = true;
	    break;
	  
	  case dods_structure_c:
	    (dynamic_cast<ClientStructure *>
	     (var(p)))->print_all_vals(os, src, dds, "", print_decl_p);
	    break;
	  
	  default:
	    // If a sequence was found, we still need to deserialize()
	    // remaining vars.
	    if(sequence_found)
		var(p)->deserialize(src, dds);
	    var(p)->print_val(os, "", print_decl_p);
	    break;
	}

	os << endl;
    }
}

AttrTable &
ClientStructure::getAttrTable()
{
    return _attr;
}

void 
ClientStructure::setAttrTable(AttrTable &attr)
{
    _attr = attr;
}
