
// (c) COPYRIGHT URI/MIT 1996
// Please read the full copyright statement in the file COPYRIGH.  
//
// Authors:
//	jhrg,jimg	James Gallagher (jgallagher@gso.uri.edu)

#ifdef __GNUG__
#pragma implementation
#endif

#include "config_writeval.h"

static char rcsid[] not_used = {"$Id: name_map.cc,v 1.6.2.3 2000/09/22 21:16:14 jimg Exp $"};

using namespace std;

#include <vector>
#include <string>
#include <strstream>

#include <Regex.h>

#include "escaping.h"
#include "name_map.h"

name_map::name_map(char *raw_equiv) 
{
    _names.push_back(name_equiv(raw_equiv));
}

name_map::name_map()
{
}

void 
name_map::add(char *raw_equiv) 
{
    name_equiv equiv(raw_equiv);
    _names.push_back(equiv);
}

/** Given a string, replace any characters that match the regular expression
    with `_<ASCII code>' where <ASCII code> is the ASCII code for the
    characters replaced. If the pattern to be replaced is a WWW escape
    sequence, then replace the % with an _ and keep the two digit ASCII code
    that follows it.

    NB: This is a modified version of esc2underscore from the dap source file
    escaping.cc. I put this here to save re-releasing the DAP yet again. It
    might ultimately belong in that source file, however.

    @param s The string to transform
    @param escape A regular expression which matches the characters to
    replace. Note: Make sure that any expression given matches only a single
    character at a time (the default regex is handled specially).
    @return The modified string. */
static string 
char2ASCII(string s, const string escape = "%[0-7][0-9a-fA-F]") {
  Regex escregx(escape.c_str(), 1);

  int i=0, matchlen;
  while ((i = escregx.search(s.c_str(), s.size(), matchlen, i)) != -1) {
    if (escape == "%[0-7][0-9a-fA-F]" && matchlen == 3)
      s.replace(i, 1, "_");
    else {
      assert(matchlen == 1);
      int ascii = *(s.substr(i, 1).data());
      ostrstream ostr;
      ostr << "_" << ascii << ends;
      s.replace(i, matchlen, ostr.str());
      ostr.freeze(0);
    }
  }

  return s;
}

static string
munge(string name)
{
  name = char2ASCII(name);
  return char2ASCII(name, "[^A-Za-z0-9_]");

}

string
name_map::lookup(string name, const bool canonical_names) 
{
  string new_name = lookup(name);
    if (!canonical_names)
	return new_name;
    else
	return munge(new_name);
}

string
name_map::lookup(string name) 
{
  // NEItor is the name_eqiv const interator. 7/23/99 jhrg
  for (NEItor p = _names.begin(); p != _names.end(); ++p)
    if (name == p->from)
      return p->to;

  return name;
}

void 
name_map::delete_all() 
{
    _names.erase(_names.begin(), _names.end());
}

#ifdef DODS_NAME_MAP_TEST
int
main(int argc, char *argv[])
{
  name_map *nm = new name_map(); 
  string res = nm->lookup("id.long", true);
  cerr << "id.long" << ": " << res << endl;

  res = nm->lookup("id$long", true);
  cerr << "id$long" << ": " << res << endl;

  res = nm->lookup("id%20long", true);
  cerr << "id%20long" << ": " << res << endl;

  delete nm;
}
#endif

// $Log: name_map.cc,v $
// Revision 1.6.2.3  2000/09/22 21:16:14  jimg
// Added char2ASCII function. Given a string, replace any characters that match
// the regular expression with `_<ASCII code>'.
// Changed the munge function so that it now calls char2ASCII() and not
// esc2underscore().
//
// Revision 1.6.2.2  2000/08/25 23:34:30  jimg
// Added a lookup(string) method that does only the thesaurus lookup and
// rewrite lookup(string, bool) to use it.
//
// Revision 1.6.2.1  2000/08/17 23:53:18  jimg
// Switched from config_dap.h to config_writeval.h.
//
// Revision 1.6  1999/07/24 00:10:07  jimg
// Repaired the munge function and removed SLList.
//
// Revision 1.5  1999/03/24 06:23:43  brent
// convert String.h to std lib <string>, convert to packages regex -- B^2
//
// Revision 1.4  1997/02/06 20:44:51  jimg
// Fixed log messages
//
// Revision 1.3  1997/01/10 06:50:30  jimg
// Added to lookup mfunc the ability to map non-alphanmerics to underscore.
//
// Revision 1.2  1996/11/23 05:17:39  jimg
// Added name_map() because g++ wants it.
//
// Revision 1.1  1996/11/22 23:52:01  jimg
// Created.
